import { __awaiter } from "tslib";
import { createObserver } from '../observer';
import { logger } from '../utils';
/**
 * An in memory database of specific DatabaseRecord types.
 * @public
 */
export class InMemoryDatabase {
    constructor(uuid) {
        this.uuid = uuid;
        this.isWorking = false;
        this.records = {};
        this.beforeUpdateListeners = createObserver();
        this.afterUpdateListeners = createObserver();
    }
    create(newValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = yield this.uuid.createRemoteId();
            return this.update(id, newValue);
        });
    }
    read(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let record;
            try {
                this.isWorking = true;
                record = this.records[id]; // cloneDeep(this.records[id]);
                logger.debug(`Database.read ${JSON.stringify(record)}`);
            }
            catch (err) {
                logger.error('Database.read error', err);
                throw err;
            }
            finally {
                this.isWorking = false;
            }
            return {
                value: record,
            };
        });
    }
    update(id, newValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const record = Object.assign(Object.assign({}, newValue), { id });
            const dataRecord = record; // cloneDeep(record) as T;
            try {
                this.isWorking = true;
                this.beforeUpdateListeners.publish({
                    value: this.records[id],
                    newValue: record,
                });
                this.records[id] = dataRecord;
                this.afterUpdateListeners.publish({
                    value: record,
                });
                logger.debug(`Database.create success ${JSON.stringify(dataRecord)}`);
            }
            catch (err) {
                logger.error('Database.create error', err);
                throw err;
            }
            finally {
                this.isWorking = false;
            }
            return {
                value: record,
            };
        });
    }
    delete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let success = true;
            try {
                this.isWorking = true;
                success = delete this.records[id];
                logger.debug(`Database.delete ${id}`);
            }
            catch (err) {
                logger.error('Database.delete error', err);
                throw err;
            }
            finally {
                this.isWorking = false;
            }
            return {
                id,
                success,
            };
        });
    }
    visit(visitor) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.isWorking = true;
                Object.values(this.records).forEach(visitor);
                logger.debug(`Database.visit complete`);
            }
            catch (err) {
                logger.error('Database.visit error', err);
                throw err;
            }
            finally {
                this.isWorking = false;
            }
        });
    }
    onBeforeUpdate(listener) {
        return this.beforeUpdateListeners.subscribe(listener);
    }
    onAfterUpdate(listener) {
        return this.afterUpdateListeners.subscribe(listener);
    }
}
