import { __decorate } from "tslib";
import { observable } from '@microsoft/fast-element';
/**
 * @public
 */
export class DefaultErrorMap {
    /**
     * @param logger - A logger error method reference.
     */
    constructor(logger) {
        this.logger = logger;
        this.map = new Map();
    }
    /** {@inheritDoc ErrorMap.set} */
    set(key, error) {
        this.logger(key, error);
        this.lastError = error;
        this.map = new Map(this.map.set(key, error));
    }
    /**
     * Get an error by key.
     * @param key - The key.
     * @returns The error if it exists, otherwise undefined.
     *
     * @public
     */
    get(key) {
        return this.map.get(key);
    }
    /**
     * Has an error for key.
     * @param key - The key.
     * @returns True if it exists, otherwise false.
     *
     * @public
     */
    has(key) {
        return this.map.has(key);
    }
    /**
     * Delete an error.
     * @param key - The key.
     * @returns True if successful, otherwise false.
     *
     * @public
     */
    delete(key) {
        const isDeletingLast = this.map.get(key) === this.lastError;
        const result = this.map.delete(key);
        if (result) {
            this.map = new Map(this.map);
            if (isDeletingLast) {
                this.lastError = undefined;
            }
        }
        return result;
    }
    /**
     * Clear errors.
     * @public
     */
    clear() {
        this.lastError = undefined;
        this.map = new Map();
    }
    /**
     * The size of the error map.
     * @returns number
     *
     * @public
     */
    get size() {
        return this.map.size;
    }
    /** {@inheritDoc ErrorMap.messages} */
    get messages() {
        const count = this.size;
        if (!count) {
            return '';
        }
        const errorMessages = [];
        for (const { message } of this.map.values()) {
            errorMessages.push(message);
        }
        return `${count} Error${count > 1 ? 's' : ''} Occurred:\n${errorMessages.join('\n')}`;
    }
}
__decorate([
    observable
], DefaultErrorMap.prototype, "map", void 0);
__decorate([
    observable
], DefaultErrorMap.prototype, "lastError", void 0);
/**
 * A factory to create the error map.
 * @param logger - A logger error method reference.
 * @returns A ErrorMap instance.
 *
 * @public
 */
export const createErrorMap = (logger) => new DefaultErrorMap(logger);
