import { __awaiter, __decorate } from "tslib";
import { observable } from '@microsoft/fast-element';
import { logger } from '../../utils';
/**
 * An enum that defines the names of the events that can be raised by the `PendingState` mixin.
 * @public
 */
export var PendingStateEvents;
(function (PendingStateEvents) {
    PendingStateEvents["state"] = "pending-state";
    PendingStateEvents["error"] = "pending-state-error";
})(PendingStateEvents || (PendingStateEvents = {}));
/**
 * The `PendingState` mixin.
 * @public
 */
export const PendingState = (Base) => {
    class Pending extends Base {
        constructor() {
            super(...arguments);
            /**
             * The number of promises that are currently pending.
             * @public
             */
            this.pendingCount = 0;
            /**
             * The number of promises that have been resolved.
             * @public
             */
            this.resolvedCount = 0;
            /**
             * A boolean indicating whether there are any pending children.
             * @public
             */
            this.hasPendingChildren = false;
        }
        /**
         * Gets the progress of the pending promises as a percentage between 0 and 1.
         * @public
         */
        get progress() {
            return this.resolvedCount / this.pendingCount;
        }
        /**
         * Called when the element is connected to the DOM.
         * @internal
         */
        connectedCallback() {
            super.connectedCallback();
            logger.debug(`PendingState mixin is now connected to the DOM`);
            /**
             * We're likely not adding this mixin to the app shell, so may need to listen on a route hub...
             */
            this.addEventListener(PendingStateEvents.state, this.onPendingState);
        }
        /**
         * Called when the element is disconnected from the DOM.
         * @internal
         */
        disconnectedCallback() {
            super.disconnectedCallback();
            logger.debug(`PendingState mixin is now disconnected from the DOM`);
            this.removeEventListener(PendingStateEvents.state, this.onPendingState);
        }
        /**
         * Handles the `pending-state` event.
         * @param event - The `pending-state` event.
         * @internal
         */
        onPendingState({ detail }) {
            return __awaiter(this, void 0, void 0, function* () {
                const { promise } = detail;
                if (promise) {
                    this.hasPendingChildren = true;
                    this.pendingCount += 1;
                    try {
                        yield promise;
                    }
                    catch (error) {
                        this.$emit(PendingStateEvents.error, {
                            message: `Error in pending-state promise: ${error}`,
                            error,
                        });
                    }
                    finally {
                        this.resolvedCount += 1;
                        if (this.pendingCount === this.resolvedCount) {
                            this.hasPendingChildren = false;
                        }
                    }
                }
            });
        }
    }
    __decorate([
        observable
    ], Pending.prototype, "hasPendingChildren", void 0);
    return Pending;
};
