import { __decorate, __param } from "tslib";
import { DI, optional } from '@microsoft/fast-foundation';
import { parse, stringify, isInteger } from 'lossless-json';
import { BIG_INT_64_BITS, JSONSerializerConfig } from './json.types';
/**
 * A Default Number Parser for deserializing objects.
 * @public
 */
export function customNumberParser(value) {
    if (Number.isSafeInteger(+value)) {
        return Number(value);
    }
    if (!isInteger(value)) {
        return parseFloat(value);
    }
    return BigInt.asIntN(BIG_INT_64_BITS, value);
}
/**
 * A Default JSONSerializer Config for serializing and deserializing functions.
 * @public
 */
export const defaultJSONSerializerConfig = {
    parse: (input) => parse(input, null, customNumberParser),
    stringify: (object) => stringify(object),
};
let DefaultJSONSerializer = class DefaultJSONSerializer {
    constructor(config = defaultJSONSerializerConfig) {
        this.config = config;
        /**
         * @inheritDoc JSONSerializer.serialize
         */
        this.serialize = (object) => this.config.stringify(object);
    }
    /**
     * @inheritDoc JSONSerializer.deserialize
     */
    deserialize(input) {
        if (!input) {
            return Promise.resolve({});
        }
        if (typeof input === 'string') {
            return this.config.parse(input);
        }
        if (input && input.ok) {
            return input.text().then((text) => this.config.parse(text));
        }
        if (input && input.data) {
            input = input.data;
        }
        return this.config.parse(input);
    }
};
DefaultJSONSerializer = __decorate([
    __param(0, optional(JSONSerializerConfig))
], DefaultJSONSerializer);
export default DefaultJSONSerializer;
/**
 * A DI token for the JSON serializer.
 * @public
 */
export const JSONSerializer = DI.createInterface((x) => x.singleton(DefaultJSONSerializer));
